﻿namespace RFTiming
{
    class Extension : RuntimeFlowExt.IExtension
    {
        public Extension()
        {
        }

        public void Init(RuntimeFlowExt.IEngine engine_)
        {
            window = new MainWindow();
            window.Show();
            formatter = new Formatter(engine_);

            threads = new System.Collections.Generic.Dictionary<uint, Thread>();
            engine = engine_;
            engine.FunctionEnter += OnFunctionEnter;
            engine.FunctionLeave += OnFunctionLeave;
            engine.FunctionsReset += OnFunctionsReset;
            engine.Clear += OnClear;
        }

        public void Close()
        {
            engine.FunctionEnter -= OnFunctionEnter;
            engine.FunctionLeave -= OnFunctionLeave;
            engine.FunctionsReset -= OnFunctionsReset;
            engine.Clear -= OnClear;
            window.Close();
        }

        private void OnFunctionEnter(uint threadID, uint functionID, uint paramsID)
        {
            System.DateTime currentTime = System.DateTime.Now;
            if (!threads.ContainsKey(threadID))
                threads.Add(threadID, new Thread());

            Thread thread = threads[threadID];
            window.AddLine(formatter.FormatFunctionCall(currentTime, threadID, functionID, paramsID, thread.GetCurrentStackDepth()));
            thread.Enter(functionID);
        }

        private void OnFunctionLeave(uint threadID, uint functionID, uint returnValueID)
        {
            threads[threadID].Leave(functionID);
        }

        private void OnFunctionsReset()
        {
            threads.Clear();
            window.Clear();
        }

        private void OnClear()
        {
            window.Clear();
        }

        MainWindow window;
        Formatter formatter;
        RuntimeFlowExt.IEngine engine;
        System.Collections.Generic.Dictionary<uint, Thread> threads;
    }
}
